/**
 * Ueberdeckung
 * 
 * This is GPL'd code
 * (c) by Patrick Kilian 2003
 *
 */
import org.vhb.jop.simpleIO.SimpleInput;
import java.lang.Math;
/**
 *
 * Eine Klasse, die die Uebderdeckung von zwei Objekten der Art Rechteck oder
 * Kreis, die vorher eingelesen werden, feststellt.
 * Das Vorhandensein der Klassen Rechteck und Kreis ist noetig.
 * Achtung es findet _keine_ Fehlerbehandlung statt.
 *
 * @author Patrick Kilian
 * @version 1.00
 */
public class Ueberdeckung {
    /**
     * Berechnet des euklidische Abstand zweier Punkte in einer Ebene
     * Achtung es findet _keine_ Fehlerbehandlung statt.
     * 
     * @param x1 x-Koordinate des ersten Punktes
     * @param y1 y-Koordinate des ersten Punktes
     * @param x2 x-Koordinate des zweiten Punktes
     * @param y2 y-Koordinate des zweiten Punktes
     * @return der euklidische Abstand
     */
    private static double dist(double x1, double y1, double x2, double y2) {
        double dist = Math.sqrt(Math.pow((x1 - x2), 2)+Math.pow((y1 - y2), 2)); 
        return (dist);
    }

    /**
     * Testet ob eine Zahl x2 zwischen x1 und x3 liegt
     * Achtung es findet _keine_ Fehlerbehandlung statt.
     *
     * @param x1 untere Grenze
     * @param x2 zu testender Wert
     * @param x3 obere Grenze
     * @return true := x2 ist nicht kleiner als x1 und nicht groesser als x3
     */
    private static boolean zwischen(double x1, double x2, double x3) {
        boolean zwischen = false;
        if ( (x1 <= x2) && (x2 <= x3)) {
            zwischen = true;
        }
        return (zwischen);
    }

    /**
     * Fuehrt die Ueberdeckungspruefung fuer zwei Kreise durch
     * Achtung es findet _keine_ Fehlerbehandlung statt.
     * keine Aufrufsparameter
     *
     * @return true := die Kreise sind nicht disjunkt
     */
    private static boolean kreis_mit_kreis() {
        boolean ret = false;
        Kreis a = new Kreis();
        Kreis b = new Kreis();
        if ( dist(a.x, a.y, b.x, b.y) <= (a.r + b.r) ) {
            ret = true;
        }
        return (ret);
    }

    /**
     * Fuehrt die Ueberdeckungspruefung fuer zwei Rechtecke durch
     * Achtung es findet _keine_ Fehlerbehandlung statt.
     * keine Aufrufsparameter
     *
     * @return true := die Rechtecke sind nicht disjunkt
     */
    private static boolean rechteck_mit_rechteck() {
        boolean ret = false;
        boolean xfit = false; // Ueberlagert sich die Projektion auf die x-Achse
        boolean yfit = false; // Ueberlagert sich die Projektion auf die y-Achse
        Rechteck a = new Rechteck();
        Rechteck b = new Rechteck();
        // Ein Eckpunkt von b liegt in a
        if ( zwischen(a.x1, b.x1, a.x2) || zwischen(a.x1, b.x2, a.x2) ) {
            xfit = true;
        }
        if ( zwischen(a.y1, b.y1, a.y2) || zwischen(a.y1, b.y2, a.y2) ) {
            yfit = true;
        }
        if (xfit && yfit) {
            ret = true;
        }
        // ODER ein Eckpunkt von a liegt in b
        if ( zwischen(b.x1, a.x1, b.x2) || zwischen(b.x1, a.x2, b.x2) ) {
            xfit = true;
        }
        if ( zwischen(b.y1, a.y1, b.y2) || zwischen(b.y1, a.y2, b.y2) ) {
            yfit = true;
        }
        if (xfit && yfit) {
            ret = true;
        }
        return (ret);
    }

    /**
     * Fuehrt die Ueberdeckungspruefung fuer einen Kreis und ein Rechteck durch
     * Achtung es findet _keine_ Fehlerbehandlung statt.
     * keine Aufrufsparameter
     *
     * @return true := die Objekte sind nicht disjunkt
     */
    private static boolean kreis_mit_rechteck() {
        boolean ret = false;
        boolean xfit = false;
        boolean yfit = false;
        boolean xfit_w = false;
        boolean yfit_w = false;
        Kreis a = new Kreis();
        Rechteck b = new Rechteck();
        if ( zwischen(b.x1, a.x, b.x2) ) {
            xfit = true;
        }
        if ( zwischen((b.x1 - a.r), a.x, (b.x2 + a.r)) ) {
            xfit_w = true;
        }
        if ( zwischen(b.y1, a.y, b.y2) ) {
            yfit = true;
        }
        if ( zwischen((b.y1 - a.r), a.y, (b.y2 + a.r)) ) {
            yfit_w = true;
        }
        if ( (xfit && yfit_w) || (xfit_w && yfit) ) {
            ret = true;
        }
        if ( dist(a.x, a.y, b.x1, b.y1) <= a.r) {
            ret = true;
        }
        if ( dist(a.x, a.y, b.x1, b.y2) <= a.r) {
            ret = true;
        }
        if ( dist(a.x, a.y, b.x2, b.y2) <= a.r) {
            ret = true;
        }
        if ( dist(a.x, a.y, b.x2, b.y1) <= a.r) {
            ret = true;
        }
        return (ret);
    }

    /**
     * Die Methode erfragt was fuer Objekte getestet werden sollen, laesst
     * diese erzeugen und laesst diese auf Ueberdeckung pruefen. Die Ausgabe
     * erfolgt als String auf die Standardausgabe.
     * @param args wird ignoriert
     */
    public static void main(String[] args) {
        boolean ueberdeckung = false;
        int type = 0;
        System.out.println("Welche Kombination mchten Sie testen?");
        System.out.println("1: Kreis mit Kreis");
        System.out.println("2: Rechteck mit Rechteck");
        System.out.println("3: Kreis mit Rechteck");
        type = SimpleInput.in.getInt();
        switch (type) {
            case 1:
                ueberdeckung = kreis_mit_kreis();
                break;
            case 2:
                ueberdeckung = rechteck_mit_rechteck();
                break;
            case 3:
                ueberdeckung = kreis_mit_rechteck();
                break;
            default:
                System.out.println("BUG: Ungueltiger type " + type);
        }
        if (ueberdeckung) {
            System.out.println("true");
        } else {
            System.out.println("false");
        }
    }
}
